"""
engine loading and configuration
"""
from ctypes import c_void_p, c_char_p, c_int
from ctypescrypto import libcrypto
from ctypescrypto.exception import LibCryptoError

__all__ = ['default', 'set_default']

default = None

def set_default(engine):
    """
    Loads specified engine and sets it as default for all
    algorithms, supported by it
    """
    global default
    eng = libcrypto.ENGINE_by_id(engine)
    if eng is None:
        # Try load engine
        eng = libcrypto.ENGINE_by_id("dynamic")
        if  eng is None:
            raise LibCryptoError("Cannot get 'dynamic' engine")
        if not libcrypto.ENGINE_ctrl_cmd_string(eng, "SO_PATH", engine, 0):
            raise LibCryptoError("Cannot execute ctrl cmd SO_PATH")
        if not libcrypto.ENGINE_ctrl_cmd_string(eng, "LOAD", None, 0):
            raise LibCryptoError("Cannot execute ctrl cmd LOAD")
    if eng is None:
        raise ValueError("Cannot find engine " + engine)
    libcrypto.ENGINE_set_default(eng, c_int(0xFFFF))
    default = eng

# Declare function result and arguments for used functions
libcrypto.ENGINE_by_id.restype = c_void_p
libcrypto.ENGINE_by_id.argtypes = (c_char_p, )
libcrypto.ENGINE_set_default.argtypes = (c_void_p, c_int)
libcrypto.ENGINE_ctrl_cmd_string.argtypes = (c_void_p, c_char_p, c_char_p,
                                             c_int)
libcrypto.ENGINE_finish.argtypes = (c_char_p, )



"""
    Interface to some libcrypto functions

"""


from ctypes import CDLL, c_char_p

def config(filename=None):
    """
        Loads OpenSSL Config file. If none are specified, loads default
        (compiled in) one
    """
    libcrypto.OPENSSL_config(filename)

__all__ = ['config']

libcrypto = CDLL("libcrypto.so.1.0.0")
libcrypto.OPENSSL_config.argtypes = (c_char_p, )
libcrypto.OPENSSL_add_all_algorithms_conf()



" wrapper around OpenSSL stack abstraction"

class Stack:
    def __init__(crytostruct,itemclass,ptr=None,items=[]):
        """
        Creates stack object.
        @param cryptostruct - name of libcrypto structure such as X509 EVP_PKEY
                        (there must exist ${cryptostruct}_free function)
        @param itemclass - python wrapper class around this structure 

        @param ptr - pointer to existing OpenSSL stack structure. If
                None, new, empty stack would be created
        @param items - list of itemclass objects. If specifed, and ptr
                is None, new stack would be populated with these itesm
                        
        """
        raise NotImplementedError

    def __len__(self):
        """
        Return number of elements in the stack
        """
        return libcrypto.sk_num(self.ptr)
    def __getitem__(self,idx):
        """
        Returns stack element by index
        @param idx - index of stack element
        """
        raise NotImplementedError
    def __putitem__(self,idx,value):
        """
        Replases the item with given index
        @param idx index to replace
        @param value itemclass object
        """
        raise NotImplementedError
    def __del__(self):
        """
        Frees memory used by stack
        """
        libcrypto.sk_pop_free(self.ptr,libcrypto[self.freefunc])
    def append(self,value):
        libcrypto.sk_push(self.ptr,itemclass.clone(value.ptr))  
